#ifndef _DHASHTABLE_H
#define _DHASHTABLE_H

// This file was automatically generated by objch.

//==============================================================================
//
//      DHashTable - the hash table collection class in the ofc-library
//
//               Copyright (C) 2003  Dick van Oudheusden
//  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public
// License along with this library; if not, write to the Free
// Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//==============================================================================
// 
//  $Date: 2005/02/05 06:25:38 $ $Revision: 1.11 $
//
//==============================================================================

#include "ofc/config.h"

#include "ofc/DDatable.h"
#include "ofc/DComparable.h"

#include "ofc/DList.h"

// The hash node is an element in a hash table

typedef struct _DHashNode
{
  id                    _key;
  id                    _object;
  struct _DHashNode    *_next;
  struct _DHashNode    *_prev;
  unsigned long         _hash;
} DHashNode;

// The hash table collection class implements a number of methods for creating, inserting
// searching and removing objects in a hash table.

@interface DHashTable : Object
{
@private
  DHashNode       **_table;    // the actual table with the nodes
  Class             _class;    // the class of the key
  unsigned long     _size;     // the size of the table
  long              _count;    // the number of nodes in the hash table
  double            _load;     // the load factor before resising >0.0
  unsigned long     _threshold;// the threshold for resizing
}


//// Constructors

//
// Initialise an empty hash table with a default size and load
//
// @param key      the class of the key (conforms DDatable and DComparable)
//
// @return the object
//
- (DHashTable *) init :(Class) key;

//
// Initialise an empty hash table with an initial size
//
// @param key       the class of the key (conforms DDatable and DComparable)
// @param size      the initial size of the table
// @param load      the load factor for resizing
//
// @return the object
//
- (DHashTable *) init :(Class) key :(unsigned long) size :(double) load;


//// Copy related methods

//
// Deepen a copy of the object (not implemented)
//
// @return the object
//
- deepen;


//// Destructor

//
// Free the hash table (the objects are *NOT* freed)
//
// @return the object
//
- shallowFree;

//
// (Deep) free the hash table (the stored objects are *FREED*)
//
// @return the object
//
- free;


//// Member methods

//
// Return the number of elements
//
// @return the length
//
- (long) length;

//
// Check if the hash table is empty
//
// @return is it?
//
- (BOOL) isEmpty;

//
// Return the load factor
//
// @return the factor
//
- (double) load;

//
// Set the load factor
//
// @param load     the new load factor (> 0.0)
//
// @return the object
//
- (DHashTable *) load :(double) load;

//
// Return the size of the hash table
//
// @return the size
//
- (unsigned long) size;

//
// Set the size of the hash table
//
// @param size     the new size for the table
//
// @return the object
//
- (DHashTable *) size :(unsigned long) size;

// printf("Resizing: %ld -> %ld for %ld\n", _size, size, _count);

//// Hash table manipulation

//
// Insert a key,object in the hash table
//
// @param key      the key to be inserted
// @param object   the object to be inserted
//
// @return success
//
- (BOOL) insert :(id) key :(id) object;

//
// Remove a key,object from the hash table
//
// @param key      the key to be removed
//
// @return the object (or nil for not found)
//
- (id) delete :(id) key;

//
// Check if the hash table has a certain key
//
// @param key      the key to be checked
//
// @return is the key present
//
- (BOOL) has :(id) key;

//
// Return the object related to the key in the hash table
//
// @param key      the key to be checked
//
// @return the object related to the key (or nil)
//
- (id) get :(id) key;

//
// Perform a method for every object in the hash table
//
// @param sel      the selector of the method
//
// @return the object
//
- (DHashTable *) each :(SEL) sel;


//// List related functions

//
// Return a list with copies of all keys in the hash table
//
// @return a (new) list with all the keys
//
- (DList *) keys;

//
// Return a list with copies of all the objects in the hash table
//
// @return a (new) list with all the objects
//
- (DList *) objects;

@end

// The DHashIterator class implements a number of methods for iterating hash tables.

@interface DHashIterator : Object
{
@private
  DHashTable     *_table;   // the table on which the iterator operates
  DHashNode      *_node;    // the current node in the list
  unsigned long   _index;   // the current index in the hash table
}


//// the constructors

//
// Initialise a hash iterator without a table
//
// @return the iterator
//
- (DHashIterator *) init;

//
// Initialises an iterator on a hash table (the iterator is moved to the first node)
//
// @param table    the hash table on which the iterator operates
//
// @return the iterator
//
- (DHashIterator *) init :(DHashTable *) table;


//// the iterator methods

//
// Set the table (iterator is moved to the first object)
//
// @return the object
//
- (DHashIterator *) hashTable :(DHashTable *) table;

//
// Get the first object in the table
//
// @return the first object (or nil)
//
- (id) first;

//
// Get the next object in the table
//
// @return the next object (or nil)
//
- (id) next;

//
// Get the previous object in the table
//
// @return the previous object (or nil)
//
- (id) prev;

//
// Get the last object in the table
//
// @return the last object (or nil)
//
- (id) last;

//
// Set the current object to another object
//
// @param object   the object to be stored in current node
//
// @return the previous stored object (or nil)
//
- (id) object :(id) object;

//
// Get the current object
//
// @return the current object (or nil)
//
- (id) object;

//
// Get the key of the current object
//
// @return a referencee to the key of the current object (or nil)
//
- (id) key;

@end
#endif
